package cd.casic.ci.api;

import cd.casic.ci.process.dto.req.sast.SastApplicationCreateReq;
import cd.casic.ci.process.dto.req.sast.SastEngineConfigReq;
import cd.casic.ci.process.dto.req.sast.SastIdentifiedLanguageTaskReq;
import cd.casic.ci.process.dto.req.sast.SastReportCreateReq;
import cd.casic.ci.process.dto.resp.sast.*;
import cd.casic.ci.process.process.service.sast.SastService;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.pojo.CommonResult;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.Negative;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

@RestController
@RequestMapping("/sast")
public class SastController {
    @Resource
    private SastService sastService;
    @GetMapping("/applicationExist")
    public CommonResult<Boolean> applicationExist(@RequestParam String applicationName){
        Boolean b = sastService.applicationExist(applicationName);
        return CommonResult.success(b);
    }
    @PostMapping("/uploadFile")
    public CommonResult<SastFileUploadResp> uploadFile(MultipartFile multipartFile){
        File tempFile = null;
        try {
            byte[] bytes = multipartFile.getBytes();
            String[] split = multipartFile.getOriginalFilename().split("\\.");
            tempFile = File.createTempFile(split[0], "."+split[1]);
            FileOutputStream fos = new FileOutputStream(tempFile);
            fos.write(bytes);
            return CommonResult.success(sastService.uploadFile(tempFile));
        } catch (IOException e) {
            return CommonResult.error(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR);
        }finally {
            if (tempFile!=null) {
                tempFile.deleteOnExit();
            }
        }
    }
    @PostMapping("/createIdentifiedLanguageTask")
    public CommonResult<SastIdentifiedLanguageTaskResp> createIdentifiedLanguageTask(@RequestBody SastIdentifiedLanguageTaskReq req){
        SastIdentifiedLanguageTaskResp identifiedLanguageTask = sastService.createIdentifiedLanguageTask(req);
        return CommonResult.success(identifiedLanguageTask);
    }
    @GetMapping("/getIdentifiedLanguageTaskStatus/{taskId}")
    public CommonResult<SastIdentifiedLanguageTaskResp> getIdentifiedLanguageTaskStatus(@PathVariable String taskId){
        SastIdentifiedLanguageTaskResp identifiedLanguageTaskStatus = sastService.getIdentifiedLanguageTaskStatus(taskId);
        return CommonResult.success(identifiedLanguageTaskStatus);
    }
    @PostMapping("/engineConfig")
    public CommonResult<List<SastEngineConfigResp>> engineConfig(@RequestBody SastEngineConfigReq req){
        return CommonResult.success(sastService.engineConfig(req.getLanguageIdList()));
    }
    @GetMapping("/detectionConfig/{languageId}")
    public CommonResult<SastDetectionConfigResp> detectionConfig(@PathVariable String languageId){
        SastDetectionConfigResp sastDetectionConfigResp = sastService.detectionConfig(languageId);
        return CommonResult.success(sastDetectionConfigResp);
    }
    @PostMapping("/applicationCreateOrUpdate")
    public CommonResult<SastApplicationCreateResp> applicationCreateOrUpdate(@RequestBody SastApplicationCreateReq req){
        SastApplicationCreateResp sastApplicationCreateResp = sastService.applicationCreate(req);
        return CommonResult.success(sastApplicationCreateResp);
    }
    @GetMapping("/getApplicationEcho/{applicationId}")
    public CommonResult<SastApplicationEchoResp> getApplicationEcho(@PathVariable String applicationId){
        SastApplicationEchoResp applicationEcho = sastService.getApplicationEcho(applicationId);
        return CommonResult.success(applicationEcho);
    }
    @PostMapping("/reportIndex")
    public CommonResult<String> reportIndex(@RequestBody SastReportCreateReq req){
        String reportId = sastService.reportIndex(req);
        return CommonResult.success(reportId);
    }
    @GetMapping("/reportDownload")
    public CommonResult<String> reportDownLoad(String reportId){
        return CommonResult.success(sastService.reportDownload(reportId));
    }

    @GetMapping("/language")
    public CommonResult<List<SastLanguageResp>> language(){
        return CommonResult.success(sastService.language());
    }
}
